/**************************************************************************
* /dev/iptrk device for Linux 2.6 kernel
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia. 
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* BH- circular queue facility
*
**************************************************************************/
#include <asm/system.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/config.h>

#include <linux/net.h>
#include <linux/socket.h>
#include <linux/sockios.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>

#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/iptrk.h>

spinlock_t    iptrk_lock = SPIN_LOCK_UNLOCKED;
atomic_t      iptrk_active_f;

iptrk_queue_t *iptrk_queue;
int           iptrk_queue_size,
              iptrk_queue_iptr,
              iptrk_queue_count,
              iptrk_overflow_count;

/**************************************************************************
*
**************************************************************************/
static void push_iptrk_sub(struct sk_buff *skb)
{
  iptrk_queue_t *p;
  struct tcphdr *thr;
  unsigned int  header_size;

  if(iptrk_queue_count == iptrk_queue_size)
  {
    iptrk_overflow_count++;
    return;
  }

  p = &iptrk_queue[iptrk_queue_iptr];
  iptrk_queue_iptr++;
  iptrk_queue_iptr %= iptrk_queue_size;
  iptrk_queue_count++;

  if(skb->dev)
    strncpy(p->dev, skb->dev->name, IPTRK_IF_SIZE);
  else
  {
    p->dev[0] = '?';
    p->dev[1] = 0;
  }
  p->ip_s = ntohl(skb->nh.iph->saddr);
  p->ip_d = ntohl(skb->nh.iph->daddr);
  p->tot_len = ntohs((unsigned int)skb->nh.iph->tot_len);
  p->protocol = skb->nh.iph->protocol;
  header_size = skb->nh.iph->ihl << 2;

  switch(p->protocol)
  {
    case IPPROTO_TCP:
    case IPPROTO_UDP:
    {
      thr = (struct tcphdr *)(skb->data + header_size);
      p->port_s = ntohs((unsigned int)thr->source);
      p->port_d = ntohs((unsigned int)thr->dest);
      break;
    }

    default:
      p->port_s = 0;
      p->port_d = 0;
      break;
  }
}

/**************************************************************************
*
**************************************************************************/
void push_iptrk(struct sk_buff *skb)
{
  spin_lock(&iptrk_lock);

 /*
  * it has to be tested within the spin lock incase module gets unlocated
  * (and queue) after the test below. if it was outside, execution could
  * enter in push_iptrk_sub with a NULL queue...
  */
  if(atomic_read(&iptrk_active_f))
    push_iptrk_sub(skb);

  spin_unlock(&iptrk_lock);
}

EXPORT_SYMBOL(iptrk_lock);
EXPORT_SYMBOL(iptrk_active_f);
EXPORT_SYMBOL(iptrk_queue);
EXPORT_SYMBOL(iptrk_queue_size);
EXPORT_SYMBOL(iptrk_queue_iptr);
EXPORT_SYMBOL(iptrk_queue_count);
EXPORT_SYMBOL(iptrk_overflow_count);
