/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* maintenance code
*
**************************************************************************/
#include <curses.h>
#include "defs.h"
#include "lib.p"
#include "iptrk.p"

/**************************************************************************
* http://vixen/cgi-bin/iptrkq_cgi.pl?
* log=%2Fvar%2Fiptrk%2Fdb_watch%2Fcurrent%2Fdb.2003.02.08.11.09&
* saddr=0.0.0.0&
* eaddr=255.255.255.255&
* test=-S&
* type=0
**************************************************************************/
PUBLIC void compose_watch_email_body(
  char *buf, char *watch_entry, char *addr)
{
  char date_str[STR_LEN + 1];

  file_date(time_st, date_str);
  strcpy(buf, "\n");
  strcat(buf, watch_entry);
  strcat(buf, "\n\n");
  strcat(buf, "Src:\t" URL_NAME);
  strcat(buf, "cgi-bin/iptrkq_cgi.pl?db=");
  strcat(buf, IPTRK_DB_WATCH_PATHC);
  strcat(buf, "/db.");
  strcat(buf, date_str);
  strcat(buf, "&saddr=");
  strcat(buf, addr);
  strcat(buf, "&eaddr=");
  strcat(buf, addr);
  strcat(buf, "&test=-S&type=0");
  strcat(buf, "&start=1970:01:01:00:00");
  strcat(buf, "&end=2038:01:19:12:00\n\n");

  strcat(buf, "Dst:\t" URL_NAME);
  strcat(buf, "cgi-bin/iptrkq_cgi.pl?db=");
  strcat(buf, IPTRK_DB_WATCH_PATHC);
  strcat(buf, "/db.");
  strcat(buf, date_str);
  strcat(buf, "&saddr=");
  strcat(buf, addr);
  strcat(buf, "&eaddr=");
  strcat(buf, addr);
  strcat(buf, "&test=-D&type=0");
  strcat(buf, "&start=1970:01:01:00:00");
  strcat(buf, "&end=2038:01:19:12:00\n");
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void treap_free_watch(treap_t **t)
{
  iptrk_t *n;

  if(*t == NULL)
    return;

  treap_free_watch(&((*t)->l));
  treap_free_watch(&((*t)->r));

  n = TREAP_ENTRY(*t);
  treap_free(&(n->oth_port_ll));
  treap_free(&(n->tcp_port_ll));
  treap_free(&(n->udp_port_ll));

  free(*t);
  *t = NULL;
}

/**************************************************************************
*
**************************************************************************/
PUBLIC int test_interval(int i)
{
  if(i == INTERVAL_D)
  {
    if(time_st->tm_mday == old_time_st.tm_mday)
      return 0;
  }
  else
    if(i == INTERVAL_M)
      if(time_st->tm_mon == old_time_st.tm_mon)
        return 0;

  return 1;
}

PUBLIC int test_zone(int zone)
{
  zone_t *p;

  int find_cmp_func(void *n)
  {
    return zone - ((zone_t *)n)->no;
  }

  p = treap_find(&(boot.zone_bt), find_cmp_func);
  if(p == NULL)
    return 0;

  if(p->day_e >= p->day_s)
  {  /*mon -> fri*/
    if(!(time_st->tm_wday >= p->day_s && time_st->tm_wday <= p->day_e))
      return 0;
  }
  else
  {  /*sun -> wed*/
    if(time_st->tm_wday < p->day_s && time_st->tm_wday > p->day_e)
      return 0;
  }

  if(p->hr_e >= p->hr_s)
  {  /*00 -> 23*/
    if(!(time_st->tm_hour >= p->hr_s && time_st->tm_hour <= p->hr_e))
      return 0;
  }
  else
  {  /*21 -> 03*/
    if(time_st->tm_hour < p->hr_s && time_st->tm_hour > p->hr_e)
      return 0;
  }

  return 1;
}

/**************************************************************************
* purge
**************************************************************************/
PRIVATE void do_purge(void)
{
  char str[STR_LEN + 1], date_str[STR_LEN + 1];

  if(!test_interval(boot.purge))
    return;

  file_date(&old_time_st, date_str);

/*watch db*/
  log_m('I', NULL, " purging db_watch");
  sprintf(str, "%s/%s", IPTRK_DB_WATCH_PATH, date_str);
  rename(IPTRK_DB_WATCH_PATHC, str);
  mkdir(IPTRK_DB_WATCH_PATHC, 0755);

/*log_ip*/
  log_m('I', NULL, " purging log_ip");
  sprintf(str, "%s/%s", IPTRK_LOG_IP_PATH, date_str);
  rename(IPTRK_LOG_IP_PATHC, str);
  mkdir(IPTRK_LOG_IP_PATHC, 0755);

/*log_port*/
  log_m('I', NULL, " purging log_port");
  sprintf(str, "%s/%s", IPTRK_LOG_PORT_PATH, date_str);
  rename(IPTRK_LOG_PORT_FILEC, str);
}

/**************************************************************************
* archive
**************************************************************************/
PRIVATE void do_archive(void)
{
  char str[STR_LEN + 1];
  int  r;

  if(!test_interval(boot.archive))
    return;

 /*
  * first run backup.sh.. this has to be done synchronously, even at
  * the risk of the ring buffer overflowing if it takes too long..
  * it's the only way it can be done i'm afraid... unless ofcourse
  * you ask all our net users to stop networking ;).
  */
  log_m('I', NULL, " backing up data");
  if(system(BACKUP_SCRIPT) == -1)
  {
    log_e(__func__, "::system()");
    return;
  }

 /*
  * run archive.sh. this stage can be done concurrently.. and
  * iptrk can go on running happerly as it's doing this
  */
  r = fork();
  if(r == -1)
  {
    log_e(__func__, "::fork()");
    exit(-1);
  }
  else
  {
   /*
    * child
    */
    if(r == 0)
      exit(system(ARCHIVE_SCRIPT));

   /*
    * parent
    */
    sprintf(str, " archiving data (created child %d)", r);
    log_m('I', NULL, str);
  }
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void check_buffer_overflow(void)
{
  FILE       *fp;
  char       str[STR_LEN + 1],
             str2[STR_LEN + 1];
  int        c;
  static int old_c;

  fp = fopen(IPTRK_PROC_PATH, "r");
  if(fp == NULL)
    return;

  for(;;)
  {
    if(feof(fp))
      break;

    if(fgets(str, STR_LEN, fp) == NULL)
      break;

    if(get_param(str2, str, "Ovfw:", STR_LEN))
    {
      if(sscanf(str2, "%u", &c) == 1)
        if(old_c != c)
        {
          old_c = c;
          strcpy(str, " kernel queue overflow !! (we're getting slaa-ammmed !)");
          log_m('W', NULL, str);
          strcpy(str2, "\n" URL_NAME "\n");
          email_operator(str, str2);
          draw_screen(1);
        }
      break;
    }
  }

  fclose(fp);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void do_maintenance(void)
{
  if(timer - old_watch_time >= boot.watch_time)
  {
    old_watch_time = timer;

   /*
    * first do purge... if we don't then we end up with
    * the first watch of the new period (month) in the log_ip
    * log's, which looks very messy
    *
    * so, when we do logging, lets make sure everything is saved
    * in the right directory !! ;)
    */
    if(time_st->tm_hour != old_time_st.tm_hour)
      do_purge();

   /*
    * do watches etc
    */
    gather_local_ips(watch_bt, &our_net_ip_bt);
    do_watch();
    do_watch_default();
    do_watch_private();
    do_dosa_address();
    do_dosa_port();
    do_log();

    save_treaps();

    treap_free(&our_net_ip_bt);
    treap_free_watch(&watch_bt);
    watch_bt_size = 0;
    watch_bt_size_lim_f = 0;

   /*
    * now we can do the rest
    */
    if(time_st->tm_hour != old_time_st.tm_hour)
    {
      do_archive();
      check_buffer_overflow();
      memcpy(&old_time_st, time_st, sizeof(struct tm));
    }
  }
}
